interface GalleryItem {
  src?: string;
}

interface GalleryFolder {
  type: 'folder';
  items: { [key: string]: GalleryItem | GalleryFolder };
}

type Gallery = { [key: string]: GalleryItem | GalleryFolder };

function deepClone(obj: Gallery | GalleryItem | GalleryFolder): Gallery | GalleryItem | GalleryFolder {
  if (obj === null || typeof obj !== 'object') {
    return obj;
  }
  let clone = Array.isArray(obj) ? [] : {};
  for (let key in obj) {
    clone[key] = deepClone(obj[key]);
  }
  return clone;
}

function createFolder(gallery: Gallery, parentFolderPath: string[], folderName: string): Gallery {
  const newGallery = deepClone(gallery) as Gallery;
  let target: Gallery | GalleryFolder = newGallery;
  for (const folder of parentFolderPath) {
    target = target[folder].items;
  }
  if (target[folderName]) {
    throw new Error('Folder already exists');
  }
  target[folderName] = { type: 'folder', items: {} };
  return newGallery;
}

function createImage(gallery: Gallery, parentFolderPath: string[], imageName: string, src: string): Gallery {
  const newGallery = deepClone(gallery) as Gallery;
  let target: Gallery | GalleryFolder = newGallery;
  for (const folder of parentFolderPath) {
    target = target[folder].items;
  }
  if (target[folderName]) {
    throw new Error('File already exists');
  }
  target[imageName] = { src };
  return newGallery;
}

function moveItems(gallery: Gallery, fromFolderPath: string[], toFolderPath: string[], items: string[]): Gallery {
  if (toFolderPath.length) {
    let parentFolder = toFolderPath[toFolderPath.length - 1];
    if (items.includes(parentFolder)) {
      throw new Error('Cannot move a folder into itself');
    }
  }

  const newGallery = deepClone(gallery) as Gallery;

  let fromFolder: Gallery | GalleryFolder = newGallery;
  for (const folder of fromFolderPath) {
    if (fromFolderPath.length && fromFolder[folder].type !== 'folder') {
      throw new Error('Bad from folder path');
    }
    fromFolder = fromFolder[folder].items || {};
  }

  let toFolder: Gallery | GalleryFolder = newGallery;
  for (const folder of toFolderPath) {
    if (toFolderPath.length && toFolder[folder].type !== 'folder') {
      console.error({ toFolderPath }, toFolder[folder]);
      throw new Error('Bad to folder path');
    }
    if (!toFolder[folder].items) {
      toFolder[folder].items = {};
    };
    toFolder = toFolder[folder].items || {};
  };

  console.log({ fromFolder, toFolder });

  items.forEach(item => {
    // avoid overwriting existing items with the same key
    if (toFolder[item]) {
      // while loop to find a unique key
      let i = 1;
      while (toFolder[item + i]) {
        i++;
      }
      toFolder[item + i] = fromFolder[item];
    } else {
      toFolder[item] = fromFolder[item];
    }
    
    delete fromFolder[item];
  });

  return newGallery;
}

function deleteItems(gallery: Gallery, folderPath: string[], items: string[]): Gallery {
  const newGallery = deepClone(gallery) as Gallery;
  let target: Gallery | GalleryFolder = newGallery;
  for (const folder of folderPath) {
    target = target[folder].items;
  }

  items.forEach(item => {
    delete target[item];
  });

  return newGallery;
}

function openFolder(gallery: Gallery, folderPath: string[]): Gallery | GalleryFolder {
  let target: Gallery | GalleryFolder = gallery;
  for (const folder of folderPath) {
    target = target[folder].items;
  }
  return deepClone(target) as Gallery | GalleryFolder;
}

function updateItem(gallery: Gallery, folderPath: string[], itemKey: string, newItem: GalleryItem): Gallery {
  const newGallery = deepClone(gallery) as Gallery;
  let target: Gallery | GalleryFolder = newGallery;
  for (const folder of folderPath) {
    target = target[folder].items;
  }
  target[itemKey] = newItem;
  return newGallery;
}

const getItem = (gallery: Gallery, folderPath: string[], id?: string): GalleryItem | GalleryFolder | null => {
  let current = deepClone(gallery) as Gallery;
  for (let i = 0; i < folderPath.length; i++) {
    const key = folderPath[i];
    if( !id && folderPath.length-1 === i){
      return current[key] as GalleryFolder | null;
    };
    if (current[key] && current[key].type === 'folder') {
      current = current[key].items || {};
    } else {
      return null; // Path is invalid
    }
  }
  return id ? current[id] || null : current as GalleryFolder | null;
};

function getItemCount(gallery: Gallery, folderPath: string[], items: string[]): number {
  let current = gallery;
  for (let key of folderPath) {
    if (current[key] && current[key].type === 'folder') {
      current = current[key].items;
    } else {
      return 0; // Path is invalid
    }
  }

  let count = 0;
  for (const key in current) {
    if (current[key].type === 'folder') {
      // If the folder is in the items list, count all items in it
      if (items.includes(key)) {
        count += countAllItems(current[key]);
      } else {
        // Otherwise, continue with normal recursive counting
        count += getItemCount(current[key].items, [], items);
      }
    } else if (items.includes(key)) {
      // Count the item if it's directly in the root and in the items list
      count++;
    }
  }

  return count;
}

// Helper function to count all items in a folder and its subfolders
function countAllItems(folder: GalleryFolder): number {
  let count = 0;
  for (const key in folder.items) {
    const item = folder.items[key];
    if (item.type === 'folder') {
      count += countAllItems(item);
    } else {
      count++;
    }
  }
  return count;
}

function getFolderStructure(gallery: Gallery): Gallery {
  const folderStructure: Gallery = {};
  for (const key in gallery) {
    const item = gallery[key];
    if (item.type === 'folder') {
      folderStructure[key] = {
        type: 'folder',
        name: item.name || key,
        items: getFolderStructure(item.items)
      };
    }
  }
  return folderStructure;
}

export {
  deepClone,
  createFolder,
  createImage,
  moveItems,
  deleteItems,
  openFolder,
  updateItem,
  getItem,
  getItemCount,
  getFolderStructure
};
