import { moveItems } from '../src';

describe('moveItems', () => {

  it('should move a single item to an existing folder in the gallery', () => {
    const initialGallery = {
      parentFolder: { type: 'folder', items: {} },
      newFolder: { type: 'folder', items: {} },
      image: { src: 'https://example.com/image.jpg' },
    }; 
    const newGallery = moveItems(initialGallery, [], ['newFolder'], ['image']);
    expect(newGallery).toHaveProperty(['newFolder', 'items']);
    expect(newGallery.newFolder.items).toHaveProperty(['image']);
    expect(newGallery.newFolder.items.image).toEqual({ src: 'https://example.com/image.jpg' });
  });

  it('should move a single item to an existing folder in the gallery, if items do not exist', () => {
    const initialGallery = {
      parentFolder: { type: 'folder' },
      newFolder: { type: 'folder' },
      image: { src: 'https://example.com/image.jpg' },
    }; 
    const newGallery = moveItems(initialGallery, [], ['newFolder'], ['image']);
    expect(newGallery).toHaveProperty(['newFolder', 'items']);
    expect(newGallery.newFolder.items).toHaveProperty(['image']);
    expect(newGallery.newFolder.items.image).toEqual({ src: 'https://example.com/image.jpg' });
  });

  it('should move multiple items to an existing folder in the gallery', () => {
    const initialGallery = {
      parentFolder: { type: 'folder', items: {} },
      newFolder: { type: 'folder', items: {} },
      image1: { src: 'https://example.com/image1.jpg' },
      image2: { src: 'https://example.com/image2.jpg' },
    }; 
    const newGallery = moveItems(initialGallery, [], ['newFolder'], ['image1', 'image2']);
    expect(newGallery).toHaveProperty(['newFolder', 'items']);
    expect(newGallery.newFolder.items).toHaveProperty(['image1']);
    expect(newGallery.newFolder.items.image1).toEqual({ src: 'https://example.com/image1.jpg' });
    expect(newGallery.newFolder.items).toHaveProperty(['image2']);
    expect(newGallery.newFolder.items.image2).toEqual({ src: 'https://example.com/image2.jpg' });
  });

  // deep nesting tests

  it('should move a folder and move all its contents', () => {
    const initialGallery = {
      firstFolder: { type: 'folder', items: {} },
      secondFolder: { type: 'folder', items: {} },
      thirdFolder: { type: 'folder', items: {
        image: { src: 'https://example.com/image.jpg' },
      } },
    }; 
    const newGallery = moveItems(initialGallery, [], ['secondFolder'], ['thirdFolder']);
    expect(newGallery).toHaveProperty('firstFolder');
    expect(newGallery).toHaveProperty('secondFolder');
    expect(newGallery.secondFolder.items).toHaveProperty(['thirdFolder']);
    expect(newGallery.secondFolder.items.thirdFolder.items.image).toEqual({ src: 'https://example.com/image.jpg' });
  });

  it('should move a folder and move all its contents, but rename if an item name already exists', () => {
    const initialGallery = {
      firstFolder: { type: 'folder', items: {} },
      secondFolder: { type: 'folder', items: {
        image: { src: 'https://example.com/image.jpg' },
      } },
      thirdFolder: { type: 'folder', items: {
        image: { src: 'https://example.com/image.jpg' },
      } },
    }; 
    const newGallery = moveItems(initialGallery, ['secondFolder'], ['thirdFolder'], ['image']);
    expect(newGallery).toHaveProperty('firstFolder');
    expect(newGallery).toHaveProperty('secondFolder');
    expect(newGallery).toHaveProperty('thirdFolder');
    expect(newGallery.thirdFolder.items.image).toEqual({ src: 'https://example.com/image.jpg' });
    expect(newGallery.thirdFolder.items.image1).toEqual({ src: 'https://example.com/image.jpg' });
  });

  it('should move a folder and move all its contents, but rename if an item name already exists, and rename again if the new name already exists', () => {
    const initialGallery = {
      firstFolder: { type: 'folder', items: {} },
      secondFolder: { type: 'folder', items: {
        image: { src: 'https://example.com/image.jpg' },
      } },
      thirdFolder: { type: 'folder', items: {
        image: { src: 'https://example.com/image.jpg' },
        image1: { src: 'https://example.com/image.jpg' },
      } },
    }; 
    const newGallery = moveItems(initialGallery, ['secondFolder'], ['thirdFolder'], ['image']);
    expect(newGallery).toHaveProperty('firstFolder');
    expect(newGallery).toHaveProperty('secondFolder');
    expect(newGallery).toHaveProperty('thirdFolder');
    expect(newGallery.thirdFolder.items.image).toEqual({ src: 'https://example.com/image.jpg' });
    expect(newGallery.thirdFolder.items.image1).toEqual({ src: 'https://example.com/image.jpg' });
    expect(newGallery.thirdFolder.items.image2).toEqual({ src: 'https://example.com/image.jpg' });
  });

  it("should fail if we're trying to move a folder into itself", ()=> {
    const initialGallery = {
      firstFolder: { type: 'folder', items: {} },
      secondFolder: { type: 'folder', items: {
        image: { src: 'https://example.com/image.jpg' },
      } },
    }; 
    
    expect(() => moveItems(initialGallery, [], ['secondFolder'], ['secondFolder'])).toThrow();
  })

  it("should fail if we're trying to move anything into existing item that is not a folder", ()=> {
    const initialGallery = {
      firstFolder: { type: 'folder', items: {} },
      secondFolder: { type: 'folder', items: {
        image: { src: 'https://example.com/image.jpg' },
      } },
    }; 
    
    expect(() => moveItems(initialGallery, [], ['image'], ['secondFolder'])).toThrow();
  });

});